program XMSDemo;
{****************************************************************************
** Unit demonstrating the use of extended memory in Turbo Pascal programs. **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************}

uses XMS, Crt;

var
  XMSHandle : Word;
  te : String;

procedure VgaScreen; assembler;
{This sets the display to VGA 320x200 in 256 colours}
asm
  mov ax, 13h
  int 10h
end;

procedure TextScreen; assembler;
{This resets the display to text mode}
asm
  mov ax, 3h
  int 10h
end;

procedure Gradient;
{This displays a smooth pattern on screen, just for demonstration purposes}
var
  Offset : Word;

begin
  for OffSet := 0 to 63999 do
    Mem [$A000:OffSet] := OffSet mod 315;
end;

procedure RedPalette;
{This sets up a smooth red palette}
var
  DAC : Byte;

begin
  for DAC := 0 to 255 do begin
    {Set up a palette...}
    Port [$3C8] := DAC;
    {Increasingly bright red}
    Port [$3C9] := DAC shr 2;
    {No green}
    Port [$3C9] := 0;
    {No blue either}
    Port [$3C9] := 0;
  end;
end;

begin
  {Switch to VGA}
  VGAScreen;

  {Set up a smooth red palette}
  RedPalette;

  {Set up the gradient}
  Gradient;

  {Assign an extended memory block 64 K-bytes in size (65536 bytes)}
  XMSGetMem (XMSHandle, 64);

  {Copy from handle 0 (conventional memory) at $A000:0000 (video memory)
  to XMS memory allocated above at offset 0 into the extended memory block
  for a total of 64000 bytes. Note that you can only copy an EVEN number
  of bytes. So a copy of 5123 bytes, for example, is not possible.}
  XMSCopy (0, $A0000000, XMSHandle, 0, 64000);

  {Wait for a keypress}
  ReadKey;

  {Clear the screen}
  fillChar (Mem[$A000:0000], 64000, 0);

  {Wait for a keypress}
  ReadKey;

  {Copy the extended memory into the video memory - this brings back the
  gradient}
  XMSCopy (XMSHandle, 0, 0, $A0000000, 64000);

  {Release the extended memory block}
  XMSFreeMem (XMSHandle);

  {Wait for a keypress}
  ReadKey;

  {Return to text}
  TextScreen;
end.